# This class takes ruby hashes and saves them to the appropriate database table.
# All model methods that have corresponding IoT interfaces have six database fields associated with them.
# The first is a field named the same as the method.  The second is name_ack, and third is name_complete, the fourth is name_timestamp, the fifth is name_status, the sixth is name_action.
# In the hash, instance is the record id (not necessarily the id column), driver is the model, interface is the method.
# Data should be saved both to the model's database table and to the corresponding log table (name_method_logs).

module Apiotics
  class Insert
    def initialize(hash = {"action" => "set-complete", "instance" => 1, "driver" => "test_driver", "interface" => "control", "value" => 72, "timestamp" => "1000000000000", "status" => "ok"})
      @action = hash["action"]
      @id = hash["instance"]
      @class = hash["driver"]
      @methods = hash["interface"]
      @value = hash["value"]
      @timestamp = hash["timestamp"]
      @status = hash["status"]
      @valid = true
      begin
        find_record
        @methods.each do |k,v|
          @record.send(("#{k}_action=").to_sym, @action)
          @type = @record.class.columns_hash[k].type
          #change_value
          @record.send("#{k}=".to_sym, v)
          @record.send("#{k}_timestamp=".to_sym, @timestamp)
          @record.send("#{k}_status=".to_sym, @status)
        end
        if @action == "set-complete"
          set_complete
        elsif @action == "set-request-ack"
          set_ack
        elsif @action == "get-ack"
          get_ack
        end
      rescue
        @valid = false
      end
    end
    
    def find_record
      begin
        klass = @class.classify.constantize
        master_klass = (klass.parent.to_s + "::" + klass.parent.to_s).classify.constantize
        master_record = master_klass.find_by(apiotics_instance: @id)
        @record = master_record.send(@class.demodulize.underscore.downcase)
      rescue Exception => e
        puts e.to_s
      end
    end
    
    def set_ack
      @methods.each do |k,v|
        @record.send("#{k}_ack=".to_sym, true)
        @record.send("#{k}_complete=".to_sym, false)
      end
    end
    
    def set_complete
      @methods.each do |k,v|
        @record.send("#{k}_ack=".to_sym, true)
        @record.send("#{k}_complete=".to_sym, true)
      end
    end
    
    def get_ack
      @methods.each do |k,v|
        @record.send("#{k}_ack=".to_sym, true)
        @record.send("#{k}_complete=".to_sym, false)
      end
    end

    def save
      @record.skip_extract = true
      @record.save
      @record.skip_extract = false
    end
    
    def action
      @action.to_s
    end
    
    def save_log
      @methods.each do |k,v|
        klass = "#{@class}_#{k}_log".classify.constantize
        r = klass.new
        r.send(("#{k}_action=").to_sym, @action)
        r.send("#{k}=".to_sym, v)
        r.send("#{k}_timestamp=".to_sym, @timestamp)
        r.send("#{k}_status=".to_sym, @status)
        r.send("#{k}_ack=".to_sym, @record.send("#{k}_ack"))
        r.send("#{k}_complete=".to_sym, @record.send("#{k}_complete"))
        r.send("#{@class.classify.demodulize.underscore.gsub("/","_")}_id=".to_sym, @record.id)
        r.save
      end
    end
    
    def valid
      return @valid
    end
    
    def change_value
      if @type == :boolean
        if @value == "true"
          @value = true
        elsif @value == "false"
          @value = false
        end
      elsif @type == :integer
        @value = @value.to_i
      elsif @type == :string
      end
    end
    
  end
end