module Apiotics
  class CreateTableGenerator < Rails::Generators::Base
    source_root File.expand_path('../templates', __FILE__)
    argument :parent, :type => :string
    argument :name, :type => :string
    argument :attributes, :type => :hash, :required => false
    
    def copy_create_table_files
      @attribute_map = {
        "smallint" => {"kind" => "integer", "limit" => 2},
        "bigint" => {"kind" => "integer", "limit" => 8},
        "uint8_t" => {"kind" => "integer", "limit" => 2},
        "uint16_t" => {"kind" => "integer", "limit" => 4},
        "uint32_t" => {"kind" => "integer", "limit" => 8},
        "int16_t" => {"kind" => "integer", "limit" => 2},
        "int32_t" => {"kind" => "integer", "limit" => 4},
        "int_64t" => {"kind" => "integer", "limit" => 8}
      }
      template "create_table.rb.erb", "db/migrate/#{date_string}_create_#{plural_name}.rb"
      unless Apiotics.configuration.local_logging == false
        sleep 1
        attributes.each do |k,v|
          @k = k
          @v = v
          template "create_logs_table.rb.erb", "db/migrate/#{date_string}_create_#{table_name}_#{@k}_logs.rb"
          sleep 1
        end
      end
    end
    
    private
    
    def date_string
      date_string = DateTime.now.strftime("%Y%m%d%H%M%S")
    end
    
    def plural_name
      parent.underscore + "_" + name.underscore.pluralize
    end
    
    def table_name
      parent.underscore + "_" + name.underscore
    end
    
    def class_name
      name.classify
    end
    
    def plural_class_name
      parent.classify + name.classify.pluralize
    end
    
    def logs_class_name(k)
      parent.classify + name.classify + k.classify + "Logs"
    end
    
    def logs_table_name(k)
      parent.underscore + "_" + name.underscore + "_" + k.underscore + "_" + "logs"
    end
    
  end
end
